//===================================CISCKER====================================
//===========================ALU Operation decoder==============================
//------------------------------------------------------------------------------
//File Name:	alu_op_decoder.v
//Purpose:		Decodes ALU operation from opcode
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module alu_operation_decoder(
			//Inputs
			input clk_0,
			input final,
			input [7:0] opcode,
			input operation,
			input carry,
			//Outputs
			output reg [5:0] alu_op
			);
			
//==========================
//		PARAMETERS
//==========================
//					   N0	
//					   |N1
//					   ||CIN
//					   |||SEL[2:0]
//					   ||||--
parameter SUB=		6'b011000;	//SUBSTRACT
parameter NEG=		6'b101000;	//NEGATE TWO'S COMPLEMENT (!A+1)
parameter ADD=		6'b000000;
parameter AND=		6'b000001;	
parameter BCLR=		6'b010001;	//CLEAR BITS SET IN MASK (A&!MASK)
parameter OR =		6'b000010;	
parameter XOR=		6'b000011;
parameter NAND=		6'b110001;
parameter NOR=		6'b110011;
parameter INC=		6'b001000;	//INCREMENT +1 (A+0+1)
parameter DEC=		6'b010000;	//DECREMENT -1 (A+!0)
parameter ASR=		6'b000101;	//ARITHMETIC SHIFT RIGHT
parameter LSR=		6'b010100;	//LOGIC SHIFT RIGHT
parameter LSL=		6'b000100;	//LOGIC SHIFT LEFT
parameter ROR=		6'b010110;	//ROTATE RIGHT
parameter ROL=		6'b000110;	//ROTATE LEFT
parameter PASS=		6'b000111;	//SKIP OPERATIONS
parameter XXX=		6'bxxxxxx;	//DON'T CARE

wire [5:0] ADC = {2'b0,carry,3'b0};  //ADD WITH CARRY (A+X+C)
wire [5:0] SBC = {2'b1,~carry,3'b0}; //SUBSTRACT WITH CARRY (A-X-C)

wire [3:0]opcode_h = opcode[7:4];
wire [3:0]opcode_l = opcode[3:0];
reg [5:0] alu_operation = 0;

always@(posedge clk_0)
if (final)
	case(opcode_l)
		4'h0:	case (opcode_h)
					4'h0,4'h1: 		alu_operation<=XXX;		//NOP,BHCC	
					4'h2:			alu_operation<=PASS;	//TSX
					4'h3,4'h4,
					4'h5,4'h6:		alu_operation<=NEG;		//NEGA,NEGB,NEG,NEG
					default:		alu_operation<=SUB;		//SUBA,SUBA,SUBA,SUBA,SUBA,SUBB,SUBB,SUBB,SUBB	
				endcase
		4'h1:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//STOP temp.value
					4'h1:			alu_operation<=XXX;		//BHCS
					4'h2:			alu_operation<=PASS;	//TXS
					4'h3,4'h4,
					4'h5,4'h6:		alu_operation<=AND;		//CLRA,CLRB,CLR,CLR	
					default:		alu_operation<=SBC;		//SBCA,SBCB
				endcase
		4'h2:	case(opcode_h)
					4'h0:			alu_operation<=DEC;		//DABNZ
					4'h1:			alu_operation<=XXX;		//BHI
					4'h2,4'h3,4'h4,
					4'h5,4'h6:		alu_operation<=INC;		//INS,INCA,INCB,INC,INC	
					default:		alu_operation<=ADD;		//ADDA,ADDB
				endcase
		4'h3:	case(opcode_h)
					4'h0: 			alu_operation<=DEC;		//DBBNZ
					4'h1: 			alu_operation<=XXX;		//BLS
					4'h2,4'h3,4'h4,
					4'h5,4'h6:		alu_operation<=DEC;		//DES,DECA,DECB,DEC
					default:		alu_operation<=AND;		//ANDA,ANDB
				endcase
		4'h4:	case(opcode_h)
					4'b0,4'b1:		alu_operation<=XXX;		//BRN,BCC
					4'h2:			alu_operation<=INC;		//INX
					4'h3,4'h4,4'h5,
					4'h6:			alu_operation<=LSR;		//LSRA,LSRB,LSR
					default:		alu_operation<=PASS;	//LDA,LDB
				endcase
		4'h5:	case(opcode_h)
					4'b0,4'b1:		alu_operation<=XXX;		//BRA,BCS
					4'h2:			alu_operation<=SUB;		//DEX
					4'h3,4'h4,4'h5,
					4'h6:			alu_operation<=ROR;		//ROR
					default:		alu_operation<=OR;		//ORA,ORB
				endcase
		4'h6:	case(opcode_h)
					4'b0,4'b1:		alu_operation<=XXX; 	//BIL,BNE
					4'h2:			alu_operation<=PASS;	//PULX
					4'h3,4'h4,4'h5,
					4'h6:			alu_operation<=ASR;		//ASRA,ASRB,ASR
					default:		alu_operation<=XOR;		//XORA,XORB
				endcase
		4'h7:	case(opcode_h)
					4'b0,4'b1:		alu_operation<=XXX; 	//BIH,BEQ
					4'h2:			alu_operation<=PASS;	//PUSHX
					4'h3,4'h4,4'h5,
					4'h6:			alu_operation<=LSL;		//LSLA,LSLB,LSL
					default:		alu_operation<=ADC;		//ADCA,ADCB
				endcase
		4'h8:	case(opcode_h)
					4'h0:			alu_operation<=SUB;		//temp.value=division
					4'b1:			alu_operation<=XXX;		//BVC
					4'h3,4'h4,4'h5,
					4'h6:			alu_operation<=ROL;		//ROLA,ROLB.ROL
					default:		alu_operation<=SUB;		//CMPA,CMPB
				endcase
		4'h9:	case(opcode_h)
					4'h0:			alu_operation<=AND;		//temp.value=multiplicacion
					4'h1:			alu_operation<=XXX;		//BVS
					4'h7,4'h8,4'h9, 	
					4'hA,4'hB:		alu_operation<=SUB;		//BITA
					default:		alu_operation<=PASS;	//PULB,TSTA,TSTB,TST,BSR,JSR
				endcase
		4'hA:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//CLV
					4'h1:			alu_operation<=XXX;		//BPL
					4'h2,4'h3:		alu_operation<=PASS;	//PUSHA,TDX
					4'h4:			alu_operation<=XXX;		//XXX
					4'h5,4'h6:		alu_operation<=OR;		//BRSET
					default			alu_operation<=PASS;	//STAA,STAB
				endcase
		4'hB:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//SEV
					4'h1:			alu_operation<=XXX;		//BMI
					4'h2:			alu_operation<=PASS;	//PSHB
					4'h3,4'h4:		alu_operation<=XXX;		//XXX
					4'h5,4'h6:		alu_operation<=BCLR;		//BRCLR
					4'h8,4'hC:		alu_operation<=ADD;		//AIS,AIX
					default:		alu_operation<=PASS;	//STS,STX
				endcase
		4'hC:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//CLC
					4'h1:			alu_operation<=XXX;		//BGE
					4'h2:			alu_operation<=PASS;	//RTS
					4'h3,4'h4:		alu_operation<=PASS;	//TBA,TAB
					4'h5,4'h6:		alu_operation<=BCLR;		//BCLR
					4'h7,4'h8:		alu_operation<=XXX;		//XXX,XXX
					4'h9,4'hA,4'hB:	alu_operation<=PASS;	//STD		
					default:		alu_operation<=SUB;		//CPX
				endcase
		4'hD:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//SEC
					4'h1:			alu_operation<=XXX;		//BLT
					4'h3,4'h4:		alu_operation<=XXX;		//XXX,XXX
					4'h5,4'h6:		alu_operation<=OR;		//BSET
					4'h7:			alu_operation<=SUB;		//SUBD
					default:		alu_operation<=PASS;	//LDS,LDX,RTI
				endcase
		4'hE:	case(opcode_h)
					4'h0:			alu_operation<=XXX;		//CLI
					4'h1:			alu_operation<=XXX;		//BGT
					4'h2:			alu_operation<=XXX;		//temp.value=WAIT
					4'h3,4'h4:		alu_operation<=SUB;		//SBA,CBA
					4'h5,4'h6:		alu_operation<=ADD;		//MOV
					default:		alu_operation<=SUB;		//CPD,SUBD,CPD
				endcase
		4'hF:case(opcode_h)
					4'h0:			alu_operation<=XXX;		//SEI
					4'h1:			alu_operation<=XXX;		//BLE
					4'h3:			alu_operation<=ADD;		//ABA
					4'h4:			alu_operation<=XXX;		//XXX
					4'h5,4'h6:		alu_operation<=PASS;	//JMP
					4'h7,4'h8,4'h9,
					4'hA,4'hB:		alu_operation<=ADD;		//ADDD
					default:		alu_operation<=PASS;	//LDD,SWI
				endcase
		default:	alu_operation<=PASS;
	endcase
		
always@(*)
if(operation)
	alu_op=alu_operation;
else
	alu_op=PASS;
	
endmodule
